function [f,H]=extract_mode_from_s4p(filename,mode)
%%% read in s parameters from four port s4p files %%%
% input 
%   filename
%   mode {s11,s21,s31,s41,...,s14,s24,s34,s44}
% output
%   f - frequency in Hz
%   H - transfer function

% Extract units and parameter types - look for the line that starts with #
[first_char]=textread(filename,'%c%*[^\n]');
headerlines=find(first_char=='#')-1;
[freq_unit,sim_type,param_type,z_term_type,z_term_value]=textread(filename,'#%s%s%s%s%f%*[^\n]',1,'headerlines',headerlines);
freq_unit=char(freq_unit);
param_type=char(param_type);
% Position to the beginign of the number array and extract number from file
headerlines=max(find(first_char=='!'));
[col0,col1,col2,col3,col4,col5,col6,col7,col8] = textread(filename,'%f%f%f%f%f%f%f%f%f','headerlines',headerlines);

freq_index=1:4:length(col0);
s1_index=1:4:length(col0);
s2_index=2:4:length(col0);
s3_index=3:4:length(col0);
s4_index=3:4:length(col0);

f=col0(freq_index)';
switch upper(freq_unit)
 case 'HZ'
      scaleF=1;
 case 'KHZ'
      scaleF=1e3;
 case 'MHZ'
      scaleF=1e6;
 case 'GHZ'
      scaleF=1e9;
 otherwise
   error('Unknown frequency unit %s',freq_unit);
   return
end
f=f*scaleF; % frequency in Hz 

switch upper(mode)
    case 'S11'
        param1=col1(s1_index)';
        param2=col2(s1_index)';
    case 'S21'
        param1=col0(s2_index)';
        param2=col1(s2_index)';
    case 'S31'
        param1=col0(s3_index)';
        param2=col1(s3_index)';
    case 'S41'
        param1=col0(s4_index)';
        param2=col1(s4_index)';
    case 'S12'
        param1=col3(s1_index)';
        param2=col4(s1_index)';
    case 'S22'
        param1=col2(s2_index)';
        param2=col3(s2_index)';
    case 'S32'
        param1=col2(s3_index)';
        param2=col3(s3_index)';
    case 'S42'
        param1=col2(s4_index)';
        param2=col3(s4_index)';
    case 'S13'
        param1=col5(s1_index)';
        param2=col6(s1_index)';
    case 'S23'
        param1=col4(s2_index)';
        param2=col5(s2_index)';
    case 'S33'
        param1=col4(s3_index)';
        param2=col5(s3_index)';
    case 'S43'
        param1=col4(s4_index)';
        param2=col5(s4_index)';
    case 'S14'
        param1=col7(s1_index)';
        param2=col8(s1_index)';
    case 'S24'
        param1=col6(s2_index)';
        param2=col7(s2_index)';
    case 'S34'
        param1=col6(s3_index)';
        param2=col7(s3_index)';
    case 'S44'
        param1=col6(s4_index)';
        param2=col7(s4_index)';
    otherwise
        error('unknown mode %s', mode);
end

switch upper(param_type)
    case 'MA'
        H=param1.*exp(j*param2*pi/180);
    case 'RI'
        H=param1+j*param2;
    case 'DB'
        H=10.^(param1/20).*exp(j*param2*pi/180);;
    otherwise
        error('unknown parameter type %s',param_type)
end


